/*
 * $Id: PageEventDispatcher.java,v 1.7 2003/05/08 00:45:56 jacobk Exp $
 */
package org.enhydra.barracuda.contrib.dbroggisch.page;

import javax.servlet.http.HttpServletRequest;
import org.enhydra.barracuda.core.event.ControlEventContext;
import org.enhydra.barracuda.core.event.DefaultEventDispatcher;
import org.enhydra.barracuda.core.event.EventBroker;
import org.enhydra.barracuda.core.event.EventContext;
import org.enhydra.barracuda.core.event.EventException;
import javax.servlet.http.HttpSession;


/**
 * The <code>PageEventDispatcher</code> is used to suppress errors that can result
 * from using the page model with session based pages. If a user makes two requests short
 * after each other it can happen that concurrent modifications of the page stored in the
 * session occur. Then for example a control event that expects a certain page to be stored in 
 * the session can get into trouble, usually resulting in classcastexceptions. 
 * <p>
 * The <code>PageEventDispatcher</code> solves this by synchronizing the request around the 
 * session object.
 * <p>
 * Future enhacements might include the possibility to turn the behaviour on or
 * off on a per-session basis.
 * 
 * @author <a href="mailto:deets@web.de">Diez B. Roggisch</a>
 * @version 1.0
 */
public class PageEventDispatcher extends DefaultEventDispatcher { 

    /**
     * <code>PAGE_SYNCHRONIZATION_ACTIVATED</code> can be used to globally
     * deactivate the synchronization. Default value is true.
     */
    public static boolean PAGE_SYNCHRONIZATION_ACTIVATED = true;
    private RenderPage _rpe = new RenderPage();

    /**
     * Synchronizes around the session if <code>PAGE_SYNCHRONIZATION_ACTIVATED</code> is true.
     *
     * @param eb an <code>EventBroker</code> value
     * @param context an <code>EventContext</code> value
     * @exception EventException if an error occurs
     */
    public void dispatchEvent(EventBroker eb, EventContext context) throws EventException {
        context.putState(DefaultEventDispatcher.DEFAULT_RESPONSE_EVENT, _rpe);
        
        if (PAGE_SYNCHRONIZATION_ACTIVATED) {
            HttpServletRequest req = ((ControlEventContext)context).getRequest();
            HttpSession session = req.getSession(false);
            if (session != null && session.getAttribute(PageFactory.PAGE_KEY) != null) {
                Object synchee = new Object();
                if (session.getAttribute(PageFactory.SYNCHEE) == null) {
                    session.setAttribute(PageFactory.SYNCHEE, synchee);
                }
                synchronized(synchee) {
                    super.dispatchEvent(eb, context);
                }
                return;
            }
        }
        super.dispatchEvent(eb, context);
    }

}
