package org.enhydra.barracuda.contrib.dbroggisch.repopulation;


import java.util.List;
import java.util.Locale;
import java.util.Iterator;
import java.util.ArrayList;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.comp.TemplateModel;
import org.enhydra.barracuda.core.forms.DefaultFormMap;
import org.enhydra.barracuda.core.forms.FormMap;
import org.enhydra.barracuda.core.forms.FormElement;
import org.enhydra.barracuda.core.forms.FormType;
import org.enhydra.barracuda.core.forms.ParseException;
import org.enhydra.barracuda.core.comp.ViewContext;
import org.enhydra.barracuda.core.comp.TemplateDirective;
import org.enhydra.barracuda.core.comp.model.ModelListener;
import org.enhydra.barracuda.core.comp.BInput;
import org.enhydra.barracuda.core.comp.BSelect;
import org.enhydra.barracuda.core.comp.BComponent;
import org.apache.log4j.*;


/**
 * The class <code>RepopulationFormMap</code> provides a convenient way to repopulate
 * HTML form contents. 
 *
 * It can be used as a TemplateModel. The values of the defined FormElements are
 * wrapped into a BInput. The BInput sets the name of the FormElement. 
 * If a prefix has been given -- either explicit with setPrefix() or implicit by the
 * overloaded map(..)-methods -- it will be prepended to the name.
 * 
 * @author <a href="mailto:deets@web.de">Diez Roggisch</a>
 * @version 1.0
 */
public class RepopulationFormMap extends ErrorFormMap implements TemplateModel {
    protected static final Logger logger = Logger.getLogger(RepopulationFormMap.class.getName());

	/**
	 * The prefix can be used to limit the 
	 *
	 */
	protected String prefix;

	/**
	 * Describe variable <code>modelName</code> here.
	 *
	 */
	protected String modelName;

    protected ViewContext viewContext = null;
    protected List listeners = new ArrayList();

	protected List _modelChilds = new ArrayList();

	public void defineElement(FormElement element) {
		super.defineElement(element);
		if(element instanceof ChildModelElement) {
			_modelChilds.add(element);
		}
	}

	public void defineElement(java.lang.String key, FormElement element) {
		super.defineElement(key, element);
		if(element instanceof ChildModelElement) {
			_modelChilds.add(element);
		}
	}

	public List getChildModels() {
		List res = new ArrayList();
		for(Iterator it = _modelChilds.iterator(); it.hasNext();) {
			ChildModelElement cme = (ChildModelElement)it.next();
			res.add(cme.getChildModel(modelName, prefix));
		}
		return res;
	}

	/**
	 * Describe <code>setPrefix</code> method here.
	 *
	 * @param prefix a <code>String</code> value
	 */
	public void setPrefix(String prefix) {
		this.prefix = prefix;
	}


	/**
	 * Describe <code>getPrefix</code> method here.
	 *
	 * @return a <code>String</code> value
	 */
	public String getPrefix() {
		return this.prefix;
	}



	// These methods simply take care of th prefix
	/**
	 * Describe <code>map</code> method here.
	 *
	 * @param req a <code>ServletRequest</code> value
	 * @return a <code>FormMap</code> value
	 */
	public FormMap map(ServletRequest req) {
		return super.map(req, prefix);
	}

	/**
	 * Describe <code>map</code> method here.
	 *
	 * @param req a <code>ServletRequest</code> value
	 * @param prefix a <code>String</code> value
	 * @return a <code>FormMap</code> value
	 */
	public FormMap map(ServletRequest req, String prefix) {
		this.prefix = prefix;
		return super.map(req, prefix);
	}

	/**
	 * Describe <code>map</code> method here.
	 *
	 * @param req a <code>ServletRequest</code> value
	 * @param loc a <code>Locale</code> value
	 * @return a <code>FormMap</code> value
	 */
	public FormMap map(ServletRequest req, Locale loc) {
		return super.map(req, prefix, loc);
	}

	// implementation of org.enhydra.barracuda.core.comp.Contextual interface

	/**
	 * Describe <code>getItem</code> method here.
	 *
	 * @param key 
	 * @return <description>
	 */
	public Object getItem(String key)
	{
		if(logger.isDebugEnabled()) {
			logger.debug("getItem() with key: " + key);
		}
		if(getElement(key) != null) {
			String name = prefix != null ? prefix + key: key;
			BComponent bc = null;
			FormElement el = getElement(key);
			if(el instanceof RepopulationElement) {
				Object res = ((RepopulationElement)el).render(getViewContext());
				if(!(res instanceof BComponent)) {
					return res;
				}
				bc = (BComponent)res;
			} 
			else {
				BInput bi = new BInput();
				if(logger.isDebugEnabled() && el.getOrigVal() != null) {
					logger.debug("Class: " + el.getOrigVal().getClass() + " Value: " + el.getOrigVal());
				}
				bi.setValue((String)el.getOrigVal());
				bc = bi;
			}
			if(bc != null) {
				bc.setName(name);
				return bc;
			}
		}
		return super.getErrorComponent(key);
	}

    public Object getItem(TemplateDirective td) {
        return getItem(td.getKeyName());
    }

	/**
	 *
	 * @param param1 <description>
	 */
	public void setViewContext(ViewContext param1)
	{
		viewContext = param1;
	}

	/**
	 *
	 * @return <description>
	 */
	public ViewContext getViewContext()
	{
		return viewContext;
	}
	// implementation of org.enhydra.barracuda.core.comp.TemplateModel interface

	/**
	 * Describe <code>getName</code> method here.
	 *
	 * @return <description>
	 */
	public String getName()
	{
		return modelName;
	}


	/**
	 * Describe <code>setName</code> method here.
	 *
	 * @param name a <code>String</code> value
	 */
	public void setName(String name)
	{
		modelName = name;
	}

	/**
	 *
	 * @param param1 <description>
	 * @return <description>
	 */
	public boolean processDirective(TemplateDirective param1)
	{
		// TODO: implement this org.enhydra.barracuda.core.comp.TemplateModel method
		return true;
	}

	// implementation of org.enhydra.barracuda.core.comp.model.Model interface

	/**
	 *
	 * @param param1 <description>
	 */
	public void addModelListener(ModelListener ml)
	{
        listeners.add(ml);
	}

	/**
	 *
	 * @param param1 <description>
	 */
	public void removeModelListener(ModelListener ml)
	{
        listeners.remove(ml);
	}

    /**
     * Forwards the given notification event to all
     * <code>TemplateModelListeners</code> that registered
     * themselves as listeners for this template model.
     *
     * @param e  the event to be forwarded
     *
     */
    public void fireModelChanged() {
        Iterator it = listeners.iterator();
        ModelListener ml = null;
        while (it.hasNext()) {
            ml = (ModelListener) it.next();
            ml.modelChanged(this);
        }
    }

}
