/**
 * $Id: DataObjectHandler.java,v 1.3 2003/05/05 16:29:39 jacobk Exp $
 */
package org.enhydra.barracuda.contrib.sam.data;

import java.io.*;
import javax.servlet.http.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.contrib.sam.xmlform.*;
import org.enhydra.barracuda.contrib.sam.models.*;
import org.apache.log4j.*;

/** Base class for other DataHandlers. DataObjectHandler instatiates the XML form
 * map specified by it's Config. If the form is already visited, validation is
 * performed.
 */
public abstract class DataObjectHandler extends DefaultBaseEventListener implements GetConfig {
    
    protected static Logger logger = Logger.getLogger(DataObjectHandler.class.getName());
    
    public abstract Config getConfig();
    protected Config config;
    
    /** <ul>
     * <li> get a DataObject based on the OID parameter given in the request
     * <li> instatiate a XMLFormMap based on a XML file given in the Config
     * <li> if the form is already visited, try to perform validation
     * <li> if validation fails, create a ErrorModel
     * <li> continue processing by calling the Config's ViewEvent
     */    
    public void handleControlEvent(ControlEventContext context) throws EventException, javax.servlet.ServletException, java.io.IOException {
        BaseEvent event = context.getEvent();
        HttpServletRequest request = context.getRequest();
        HttpSession session = request.getSession();
        
        config = getConfig();
        
        if (logger.isDebugEnabled()) logger.debug("handling controlEvent " + event.getClass().getName());
        DataObject dataObj = null;
        try {
            dataObj = config.getDataObject(request.getParameter(config.getOidName()));
        }	catch (DataObjectException ex) {
            logger.error(ex.getMessage() );
        }

		  controlEventHookBeforeXmlFormMap(context, dataObj);
        
        XmlFormMap xfm = new XmlFormMap(context, config.getXmlFormName(), config.getFormName(), dataObj);
        context.putState(config.getMapName(), xfm);
        if (logger.isDebugEnabled()) logger.debug("putting session attribute " + config.getMapName() + " value: " + xfm);
        
        if (config.getFormName().equals(request.getParameter("visited"))) {
            try {
                logger.debug("Mapping form");
                xfm.map(request).validate(true);
            } catch (ValidationException ex) {
                ErrorModel.create(context, ex);
                logger.error(ex.getMessage(), ex);
                throw new InterruptDispatchException(config.getViewEvent());
            }
        }
		  controlEventHookAfterXmlFormMap(context, dataObj);
        
        ViewEvent renderEvent = config.getViewEvent();
        if (logger.isDebugEnabled()) logger.debug("putting " + renderEvent.getClass().getName() + " to event queue");
        event.setHandled(true);
        context.getQueue().addEvent(renderEvent);
    }

	/** hook funtion to be overridden in subclasses, empty implementation. It is called before the XMLFormMap is instantiated */
	protected void controlEventHookBeforeXmlFormMap(ControlEventContext context, DataObject dataObj) {
	}

	/** hook funtion to be overridden in subclasses, empty implementation. It is called after the XMLFormMap is instantiated and validated */
	protected void controlEventHookAfterXmlFormMap(ControlEventContext context, DataObject dataObj) {
	}
}
