/*
 * IterativeResultSetMapModel.java
 *
 * Created on 9. Dezember 2002, 14:43
 */

package org.enhydra.barracuda.contrib.sam.models;

import java.util.*;
import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.model.*;

//import org.enhydra.barracuda.core.event.*;

/** Abstract Decorator class for using cascading models.
 * It implements the well known decorator pattern.
 * This class is subclassed by concrete Decorators, see {@link MapDecorator}.<br>
 * TemplateModelDecorator is capable of handling {@link
 * org.enhydra.barracuda.core.comp.TemplateModel } as well as {@link
 * org.enhydra.barracuda.core.comp.IterativeModel }. <br>
 * <B>Implementation detail:</B>
 * Nearly all calls are passed directly to the decorated model. If the decorated
 * model implements {@link org.enhydra.barracuda.core.comp.IterativeModel }, also
 * all iterative methods are passed through.<br>
 * Subclasses should override getItem, in order to modify the standard behavior.
 * @author Stefan Armbruster
 * @version $Id: TemplateModelDecorator.java,v 1.3 2003/03/07 20:19:19 christianc Exp $
 */
public abstract class TemplateModelDecorator implements TemplateModel, IterativeModel {
    
    /** reference to the decorated model */    
    protected TemplateModel _templateModel;
    
    public TemplateModelDecorator(TemplateModel tm) {
        _templateModel = tm;
    }
    
    public String getName() {
        return _templateModel.getName();
    }
    
    public boolean processDirective(TemplateDirective td) {
        return _templateModel.processDirective(td);
    }

    //csc_030703.1 - added    
    public Object getItem(TemplateDirective td) {
        return _templateModel.getItem(td);
    }

//    abstract public Object getItem(String key);
    
    
    
    // Implementation of Model interface
    public void addModelListener(ModelListener ml)  {
        _templateModel.addModelListener(ml);
    }
    
    public void removeModelListener(ModelListener ml) {
     _templateModel.removeModelListener(ml);
    }   
    
    // implementation of Contextual interface
    public ViewContext getViewContext() {
        return _templateModel.getViewContext();
    } 
    
    public void setViewContext(ViewContext vc) {
        _templateModel.setViewContext(vc);
    }

    // implementation of IterativeModel
    
    private boolean isIterativeModel() {
        return _templateModel instanceof IterativeModel;
    }
    
    public boolean hasNext() {
        if (isIterativeModel()) {
            return ((IterativeModel)_templateModel).hasNext();
        } else {
            return false;
        }
    }
    
    public void loadNext() {
        if (isIterativeModel()) {
            ((IterativeModel)_templateModel).loadNext();
        }
    }
    
    public void postIterate() {
        if (isIterativeModel()) {
            ((IterativeModel)_templateModel).postIterate();
        }
    }
    
    public void preIterate() {
        if (isIterativeModel()) {
            ((IterativeModel)_templateModel).preIterate();
        }
    }
    
}
