package org.enhydra.barracuda.contrib.sam.examples.data;

import org.enhydra.barracuda.contrib.sam.data.*;
import org.enhydra.barracuda.core.comp.*;
import java.util.*;
import org.apache.log4j.*;

/** User is a sample class for some data object. For demo purposes it stores all
 * instances of itself in a list, normally we would use a database for this.
 */
public class User implements DataObject {
    protected static Map USERS = new HashMap();
    protected static int maxOid = 0;
    
    protected static Logger logger = Logger.getLogger(User.class.getName());
    
    protected String oid;
    protected String login;
    protected String password;
    protected String email;
    protected List roles;
    
    public User()
    throws DataObjectException {
        oid = Integer.toString(maxOid++);
        logger.debug("consturcting new USER");
    }
    
    public Object get(String field)
    throws DataObjectException {
        field = field.toLowerCase();
        if (field.equals("login")) return login;
        if (field.equals("password")) return password;
        if (field.equals("email")) return email;
        if (field.equals("oid")) return oid;
        if (field.equals("fk_roles")) return roles;
        //if (field.equals("fk_serviceprovider")) return getFk_serviceprovider();
        throw new DataObjectException("invalid field " + field);
    }
    
    public void set(String field, Object value)
    throws DataObjectException {
        field = field.toLowerCase();
        if (field.equals("login")) { login = (String)value; return; }
        if (field.equals("password")) { password = (String)value; return; }
        if (field.equals("email")) { email=(String)value; return; }
        if (field.equals("fk_roles")) {
            if (value instanceof List) {
                roles = (List)value;
            } else if (value instanceof String) {
                roles = new ArrayList();
                roles.add(value);
            }
            return;
        }
        if (field.equals("oid")) { 
            // oid is only set during construction, so we ignore this one
            return; 
        }
        throw new DataObjectException("invalid (oid is " + this.oid + ") field " + field + " " + value);
    }
    
    
    /**
     * save a user
     */
    public void save(Object dbt)
    throws DataObjectException {
        USERS.put(this.oid, this);
    }
    
    /**
     * Lscht den Benutzer aus der Datenbank
     */
    public void delete(Object dbt)
    throws DataObjectException {
        USERS.remove(this.oid);
    }
    
    public boolean equals(User cmp) {
        return (this.oid == cmp.oid);
    }
    
    public static User getUser(String oid)  throws DataObjectException {
        User ret = (User) USERS.get(oid);
        if (ret == null) {
            ret = new User();
        }
        return ret;
    }
    
    public static Collection getAll() {
        return USERS.values();
    }
    
    public static List getFormMapRoles() {
        List result = new ArrayList();
        result.add(new DefaultItemMap("1","Admin"));
        result.add(new DefaultItemMap("2", "Poweruser"));
        result.add(new DefaultItemMap("3", "regular User"));
        result.add(new DefaultItemMap("4", "DAU"));
        return result;
    }
    
    
}
