/**
 * $Id: UserGateway.java,v 1.2 2003/05/06 21:46:20 jacobk Exp $
 */
package org.enhydra.barracuda.contrib.sam.examples.data;

import java.util.*;
import javax.servlet.http.*;
import javax.servlet.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.helper.*;
import org.enhydra.barracuda.contrib.dbroggisch.display.*;
import org.enhydra.barracuda.contrib.sam.xmlform.*;
import org.enhydra.barracuda.contrib.sam.data.*;
import org.enhydra.barracuda.contrib.sam.models.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.contrib.sam.examples.events.*;
import org.enhydra.barracuda.contrib.sam.examples.xmlc.*;

public class UserGateway extends DataObjectGateway {
    
    //public constants
    protected static Logger logger = Logger.getLogger(UserGateway.class.getName());
    protected Class getDefaultEvent() { return org.enhydra.barracuda.contrib.sam.examples.events.User.class; }
    protected Class getDefaultHandler() { return UserHandler.class; }
    protected Class getDeleteEvent() { return UserDelete.class; }
    protected Class getDeleteHandler() { return UserDeleteHandler.class; }
    protected Class getListEvent() { return UserList.class; }
    protected Class getListHandler() { return UserListHandler.class; }
    protected Class getNewEvent() { return UserNew.class; }
    protected Class getNewHandler() { return UserNewHandler.class; }
    protected Class getSaveEvent() { return UserSave.class; }
    protected Class getSaveHandler() { return UserSaveHandler.class; }
    protected Class getRenderEvent() { return RenderUser.class; }
    protected Class getRenderHandler() { return RenderUserHandler.class; }
    
    /**
     * Public constructor
     */
    public UserGateway() {
        super();
        logger.info("registering UserGateway's listeners");
        specifyLocalEventInterests(new EventConnectorFactory(UserListHandler.class), UserList.class);
        specifyLocalEventInterests(new EventConnectorFactory(RenderUserListHandler.class), RenderUserList.class);
        specifyLocalEventInterests(new EventConnectorFactory(UserSaveEmailHandler.class), UserSaveEmail.class);
    }
    
    public static class UserConfig implements Config {
        
        public DataObject getDataObject(String oid)  throws DataObjectException {
            return org.enhydra.barracuda.contrib.sam.examples.data.User.getUser(oid);
        }
        
        public String getFormName() { return "user";}
        public String getMapName() { return "map"; }
        public String getOidName() { return "oid"; }
        public ViewEvent getViewEvent() { return new RenderUser(); }
        public String getXmlFormName() { return "/WEB-INF/userform.xml"; }
    }
    
    public static class UserNewHandler extends DataObjectNewHandler {
        public Config getConfig() { return new UserConfig(); }
    }
    
    public static class UserDeleteHandler extends DataObjectDeleteHandler {
        public Config getConfig() { return new UserConfig(); }
    }
    
    public static class UserSaveHandler extends DataObjectSaveHandler {
        public Config getConfig() { return new UserConfig(); }
        protected Object modifyFormData(String key, Object value ) {
            if (key.equals("password") && ((value==null) || ((String)value).length()==0)) return "sample";
            return value;
        }
        
    }
    
    public static class UserSaveEmailHandler extends DefaultBaseEventListener {
        protected static Logger logger = Logger.getLogger(UserSaveEmailHandler.class.getName());
        public void handleControlEvent(ControlEventContext context) throws EventException, javax.servlet.ServletException, java.io.IOException {
            Config config = new UserConfig();
            
            ErrorModel.create(context, new Exception("the email feature is currently not implemented. This is just an example how you can override default behavior of the SaveHandler."));
            // placeholder for additional code when saving a dataobject e.g. send email with credentials....
            
        }
    }
    
    public static class UserHandler extends DataObjectHandler {
        public Config getConfig() { return new UserConfig(); }
    }
    
    public static class RenderUserHandler extends DataObjectViewHandler {
        
        public Config getConfig() { return new UserConfig(); }
        
        public Object getTemplateModels(ViewContext vc) {
            List l = (List)super.getTemplateModels(vc);
            l.add(new ErrorModel(vc));
            HashMapModel hhm = new HashMapModel("Submit");
            hhm.setItem("Save", new BAction(new UserSave()));
            hhm.setItem("Delete", new BAction(new UserDelete()));
            hhm.setItem("New", new BAction(new UserNew()));
            hhm.setItem("SaveEmail", new BAction(new UserSaveEmail()));
            l.add(hhm);
            return l;
        }
        
        public Class getTemplateClass(ViewContext vc) {return userHTML.class;}
    }
    
    public static class UserListHandler extends DefaultBaseEventListener {
        
        public void handleControlEvent(ControlEventContext context) throws EventException, javax.servlet.ServletException, java.io.IOException {
            BaseEvent event = context.getEvent();
            HttpServletRequest request = context.getRequest();
            HttpSession session = request.getSession();
            
            event.setHandled(true);
            context.getQueue().addEvent(new RenderUserList());
        }
    }
    
    public static class RenderUserListHandler extends BTemplateViewHandler {
        public Object getTemplateModels() { return new UserListModel(); }
        public Class getTemplateClass() {return userlistHTML.class;}
    }
    
    public static class UserListModel extends IterativeCollectionModel {
        
        public UserListModel() {
            super("Nav");
            Collection l = org.enhydra.barracuda.contrib.sam.examples.data.User.getAll();
            setModels(l);
        }
        
        public Object getItem(String key) {
            try {
                org.enhydra.barracuda.contrib.sam.examples.data.User user = (org.enhydra.barracuda.contrib.sam.examples.data.User)_current;
                if (key.equals("Show")) {
                    BLink r = new BLink((String)user.get("login"), new org.enhydra.barracuda.contrib.sam.examples.events.User());
                    r.setParam("oid", (String)user.get("oid"));
                    return r;
                }
                else return super.getItem(key);
            } catch (DataObjectException ex) {
                logger.error(ex.getMessage(),ex);
                return ex.getMessage();
            }
        }
    }
}
