package org.enhydra.barracuda.contrib.sam.xmlform;

import java.util.*;
import java.io.*;
import javax.servlet.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.contrib.sam.xmlform.dtd.*;
import org.apache.log4j.*;

/**
* XmlFormFactory holds a map of parsed forms. This way, the number of xml parsings is reduced as much as possible
*/
public class XmlFormFactory {

	protected static Logger logger = Logger.getLogger(XmlFormFactory.class.getName());

   /** Map holding all already parsed XMLForms */   
	protected static Map parsedXmlFiles = new Hashtable();
   
   /** Hidden constructor to prevent any instantiazion */   
   protected XmlFormFactory() {}

   /** If the requested Form is already stored in {@link #parsedXmlFiles} it is simply
    * returned. Otherwise the XML file is stored in parsed form. Parsing is done by
    * Zeus generated classes.
    */   
	public static Form get(ControlEventContext context, String filename) 
	{
		try {
			Form form = (Form) parsedXmlFiles.get(filename);
			if (form == null) {
				if (logger.isDebugEnabled()) logger.debug(filename + " is currently not in map, trying to load");
				ServletConfig config = context.getConfig();
				ServletContext sc	= config.getServletContext();
				if (logger.isDebugEnabled()) logger.debug( "URL is " + sc.getResource(filename));
				InputStream is = sc.getResourceAsStream(filename);
				form = FormImpl.unmarshal(is);
				parsedXmlFiles.put(filename, form);
			}
			return form;
		} catch (Exception ex) {
			logger.error(ex.getMessage(),ex);
			return null;
		}
	}

} 
