/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultItemMap.java,v 1.6 2002/07/16 19:16:40 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;


/**
 * An ItemMap can be returned by the getItem() methods within any of 
 * the model implementations. It is used to associate a key with a 
 * particular value. It is primarily used by the BSelect component 
 * (which can use both key and value information when rendering). 
 * The other components just use the value information.
 */
public class DefaultItemMap implements ItemMap {

    protected Object key = null;
    protected Object value = null;

    /**
     * Create a new DefaultItemMap with the given key
     * and value
     *
     * @param key an int key
     * @param value the object value
     */
    public DefaultItemMap(int ikey, Object ivalue) {
        key = new Integer(ikey);
        value = ivalue;
    }

    /**
     * Create a new DefaultItemMap with the given key
     * and value
     *
     * @param key an Integer key
     * @param value the object value
     */
    public DefaultItemMap(Object ikey, Object ivalue) {
        key = ikey;
        value = ivalue;
    }

    /**
     * Returns the key corresponding to this entry.
     */
    public Object getKey() {
        return key;
    }

    /**
     * Returns the value corresponding to this entry. 
     */           
    public Object getValue() {
        return value;
    }

    /**
     * Replaces the value corresponding to this entry with the specified 
     * value (optional operation). 
     */          
    public Object setValue(Object ivalue) {
        value = ivalue;
        return value;
    }

    /**
     * Return a String representation of the ItemMap
     */    
    public String toString() {
//csc_071602.1 - modify so that if there is a null value in the item map we don't get a null pointer exception here
//csc_071602.1        return value.toString();
        return (value!=null ? value.toString() : "null");   //csc_071602.1
    }

}