/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ListSelectionModel.java,v 1.6 2003/05/06 03:56:32 jacobk Exp $
 */
package org.enhydra.barracuda.core.comp;

import org.enhydra.barracuda.core.comp.model.Model;


/**
 * This interface defines the methods needed to implement a 
 * ListSelectionModel. Note that this comes almost directly 
 * from javax.swing.ListSelectionModel. The only thing that 
 * is different is that we've dropped the isAdjusting stuff.
 */
public interface ListSelectionModel extends Model {

    /**
     * A value for the selectionMode property: select one list index
     * at a time.
     * 
     * @see #setSelectionMode
     */
    public static final int SINGLE_SELECTION = 0;

    /**
     * A value for the selectionMode property: select one contiguous
     * range of indices at a time.
     * 
     * @see #setSelectionMode
     */
    public static final int SINGLE_INTERVAL_SELECTION = 1;

    /**
     * A value for the selectionMode property: select one or more 
     * contiguous ranges of indices at a time.
     * 
     * @see #setSelectionMode
     */
    public static final int MULTIPLE_INTERVAL_SELECTION = 2;

    public void setSelectionMode(int selectionMode);
    public int getSelectionMode();
    public void setSelectionInterval(int index0, int index1);
    public void addSelectionInterval(int index0, int index1);
    public void removeSelectionInterval(int index0, int index1);
    public int getMinSelectionIndex();
    public int getMaxSelectionIndex();
    public boolean isSelectedIndex(int index);
    public boolean isSelectionEmpty();
    public void clearSelection();

}
