/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BTemplateGateway.java,v 1.9 2003/01/21 05:42:39 jacobk Exp $
 */
package org.enhydra.barracuda.core.comp.helper;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.log4j.*;
import org.w3c.dom.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.dom.*;

/**
 * <p>A custom implementation of the component gateway tailored for
 * template components. All you have to do is instantiate the class
 * with the name of the template and a reference to the model; it takes 
 * care of everything else.
 */
public abstract class BTemplateGateway extends ComponentGateway {

    //public vars...eventually, these should probably be final
    protected static final Logger logger = Logger.getLogger(BTemplateGateway.class.getName());
    

    //-------------------- TemplateGateway -----------------------
    /**
     * Return an instance of the template model
     */
    public abstract TemplateModel getTemplateModel();
    
    /**
     * Return an instance of the template class (must implement Document)
     */
    public abstract Class getTemplateClass();
    
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //load the localized DOM template
//csc_061702.1        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(getTemplateClass());
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(getTemplateClass(), vc.getViewCapabilities());    //csc_061702.1

        //create a template component and bind it to the views
//        TemplateView tv = new DefaultTemplateView(page.getDocumentElement());
//        BTemplate templateComp = new BTemplate(getTemplateModel(), tv);
        BTemplate templateComp = new BTemplate(getTemplateModel());
        templateComp.setView(new DefaultTemplateView(page.getDocumentElement()));
        root.addChild(templateComp);

        //return the page
        return page;
    }


}
