/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLToggleRenderer.java,v 1.9 2003/01/21 05:42:39 jacobk Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;

import org.apache.log4j.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.plankton.*;

/**
 * This class handles the default rendering of a list in an HTML view.
 */
public class HTMLToggleRenderer extends HTMLInputRenderer {

    protected static final Logger logger = Logger.getLogger(HTMLToggleRenderer.class.getName());
    
    protected BToggleButton btbcomp = null;
    
    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a list component
        if (!(comp instanceof BToggleButton)) throw new NoSuitableRendererException("This renderer can only render BToggleButton components; comp is of type:"+comp.getClass().getName());
        btbcomp = (BToggleButton) comp;
        Node node = view.getNode();

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //..HTMLInputElement - set the "selected" attribute
        if (node instanceof HTMLInputElement) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on HTMLInputElement interface...");
            HTMLInputElement el = (HTMLInputElement) node;

            //set selected
            el.setChecked(btbcomp.isSelected());

        } else {
            String errmsg = "Node does not implement HTMLInputElement and cannot be rendered: "+node;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
    }
}