/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XMLTextRenderer.java,v 1.11 2003/01/21 05:42:39 jacobk Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.xml;

import java.util.*;

import org.apache.log4j.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.plankton.*;

/**
 * This class handles the default rendering of text into an XML view.
 */
public class XMLTextRenderer extends XMLComponentRenderer {

    protected static final Logger logger = Logger.getLogger(XMLTextRenderer.class.getName());

    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a text component
        if (!(comp instanceof BText)) throw new NoSuitableRendererException("This renderer can only render BText components");
        BText btext = (BText) comp;
        Node node = view.getNode();
        String text = btext.getText();                                      //csc_121201.1

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //Element Interface - get the first Text child and set value
        //----------------------
        if (node instanceof Element) {
            if (logger.isInfoEnabled()) logger.info("Rendering based on Element interface...");
//csc_121201.1            DOMUtil.getFirstText((Element) node).setData(btext.getText());
            if (text!=null) DOMUtil.setTextInNode((Element) node, text, btext.allowMarkupInText());  //csc_121201.1
        } else {
            String errmsg = "Node does not implement Element and cannot be rendered: "+node;
            logger.warn(errmsg);
            throw new NoSuitableRendererException(errmsg);
        }
    }
    
}