/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: AbstractFormValidator.java,v 1.13 2003/01/21 05:42:53 jacobk Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.plankton.*;
import org.apache.log4j.*;

/**
 * The root implementation of the FormValidator interface
 */
public abstract class AbstractFormValidator implements FormValidator {

    protected static final Logger localLogger = Logger.getLogger(AbstractFormValidator.class.getName());

    private String myErrMsg = null;

    /**
     * Set the error message to be used in the event of an error.
     *
     * @param errmsg the error message to be used in the event
     *        of an error
     */
    public void setErrorMessage(String ierrmsg) {
        this.myErrMsg = ierrmsg;
    }

    /**
     * Get the error message to be used in the event of an error.
     *
     * @return the error message to be used in the event
     *        of an error
     */
    public String getErrorMessage() {
        return this.myErrMsg;
    }

    /**
     * This is a simple convenience method to return the appropriate type
     * of exception based on whether we're deferring or not.
     *
     * @param source     the source of the exception
     * @param deferExceptions
     *                   true if we want to generate a DeferredValidationException
     * @param defaultMsg the error message to use if no error message specified in
     *                   the constructor of the validator
     * @return a validation exception 
     */
    public ValidationException generateException(Object source, boolean deferExceptions, String defaultMsg) {
        if (localLogger.isDebugEnabled()) localLogger.debug("Generating exception... defer="+deferExceptions);
        String errmsg = (null==this.getErrorMessage()) ? defaultMsg : this.getErrorMessage();
        if (deferExceptions) return new DeferredValidationException(source, errmsg);
        else return new ValidationException(source, errmsg);
    }

    // ilc_02212002 start
    // added new method to check for nullness
    /**
     * Check if val passed is null in a consistant manner.
     * Most validators should call isNull(val) first thing
     * and return if that is the case.  Leave null validation
     * up to NotNullValidator.
     *
     * @param val the value to test for nullness
     */
    public boolean isNull(Object val, FormElement element){
      return ((val==null) || (val.toString().trim().length() < 1));
    }
    // ilc_02212001 End

}
