/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultDOMFactory.java,v 1.5 2002/10/31 06:33:47 JacobK Exp $
 */
package org.enhydra.barracuda.core.util.dom;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;

import org.enhydra.xml.xmlc.*;

/**
 * Default implementation of a DOMFactory. This class will load a DOM using
 * XMLCFactory.
 */
public class DefaultDOMFactory implements DOMFactory {

    /**
     * Get a new instance of the DOM that is associated with the
     * given class.
     *
     * @param cl the class of DOM to be loaded. In this case, this class
     *        should implement XMLObject or the underlying XMLCFactory will
     *         not be able to instantiate it.
     * @return the document that most closely corresponds with the requested
     *        class/ViewCapabilities combination.
     */
    public Document getInstance(Class cl) throws IOException {
        if (!((XMLObject.class).isAssignableFrom(cl))) throw new IOException ("Class "+cl.getName()+" can not be loaded by this DOMFactory because it does not implement XMLOBject");
        XMLCFactory xmlcFactory = new XMLCStdFactory(cl.getClassLoader(), null);
        XMLObject page = xmlcFactory.create(cl);
        return page;
    }
}