/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XMLCDeferredParsingDOMFactory.java,v 1.1 2003/04/11 19:35:24 jacobk Exp $
 */
package org.enhydra.barracuda.core.util.dom;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.deferredparsing.XMLCDeferredParsingFactory;


/**
 * XMLC deferred parsing implementation of a DOMFactory. This class will load a
 * DOM using XMLCDeferredParsingFactory.
 */
public class XMLCDeferredParsingDOMFactory implements DOMFactory {

    /**
     * Get a new instance of the DOM that is associated with the
     * given class.
     *
     * @param clazz the class of DOM to be loaded. In this case, this class
     *        should implement XMLObject or the underlying XMLCFactory will
     *        not be able to instantiate it.
     * @return the document that most closely corresponds with the requested
     *         class/ViewCapabilities combination.
     */
    public Document getInstance(Class clazz) throws IOException {
        if (!((XMLObject.class).isAssignableFrom(clazz))) throw new IOException ("Class "+clazz.getName()+" can not be loaded by this DOMFactory because it does not implement XMLOBject");
        XMLCFactory xmlcFactory = new XMLCDeferredParsingFactory(null, clazz.getClassLoader(), null);
        return xmlcFactory.create(clazz);
    }

}
