/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: LoginServices.java,v 1.3 2002/11/02 19:15:53 cryd0221 Exp $
 */
package org.enhydra.barracuda.examples.ex1;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.plankton.http.*;

/**
 * This class provides basic Login services, specifically:
 * - validateUserPwd (either from HttpReq, HttpSession,
 *   or based on a specific user/pwd combo)
 */
public class LoginServices {

    //session constants used throughout the app
    public static final String NUMBER_OF_VISITS = LoginServices.class.getName()+".NumberOfVisits";    //int
    public static final String AUTO_LOGIN = LoginServices.class.getName()+".AutoLogin";                //boolean
    public static final String USER = "User";            //String
    public static final String PASSWORD = "Password";    //String

    //validateUserPwd results
    public static final int VALID = 0;
    public static final int INVALID_USER = 1;
    public static final int INVALID_PASSWORD = 2;
    public static final int INVALID_DATA_MISSING = 3;
    public static final int INVALID_SESSION_EXPIRED = 4;
    public static final int INVALID_OTHER = 99;

    /**
     * Convert validation results into text
     */
    public static String xrefValidity(int valid) {
        if (valid==VALID) return "VALID";
        else if (valid==INVALID_USER) return "INVALID_USER";
        else if (valid==INVALID_PASSWORD) return "INVALID_PASSWORD";
        else if (valid==INVALID_DATA_MISSING) return "INVALID_DATA_MISSING";
        else if (valid==INVALID_SESSION_EXPIRED) return "INVALID_SESSION_EXPIRED";
        else if (valid==INVALID_OTHER) return "INVALID_OTHER";
        else return "???";
    }

    /**
     * Validate user/pwd, getting the information from the
     * HttpServletRequest object
     */
    public static int validateUserPwd(HttpServletRequest req, boolean updateSession) throws EventException {
        //get the login information from the request
        String user = req.getParameter(USER);
        String password = req.getParameter(PASSWORD);

        //update this info in the session if requested
        if (updateSession) {
            HttpSession session = SessionServices.getSession(req);
            session.setAttribute(USER, ""+user);
            session.setAttribute(PASSWORD, ""+password);
        }

        //this is kind of a hokey little way to catch our exception (which is just
        //for demo purposes anyway) and clean up after ourselves by removing the
        //user info fromt he session, so the error does not continue indefinitely)
        int result = INVALID_OTHER;
        try {result = validateUserPwd(user, password);}
        catch (WhoYouCallingKlausException e) {
            if (updateSession) {
                HttpSession session = SessionServices.getSession(req);
                session.setAttribute(USER, null);
                session.setAttribute(PASSWORD, null);
            }
            throw e;
        }
        return result;
    }

    /**
     * Validate user/pwd, getting the information from the
     * HttpSession object
     */
    public static int validateUserPwd(HttpSession session) throws EventException {
        //make sure the session hasn't expired


        //get the login information from the request
        String user = (String) session.getAttribute(USER);
        String password = (String) session.getAttribute(PASSWORD);
        return validateUserPwd(user, password);
    }

    /**
     * Validate user/pwd
     */
    public static int validateUserPwd(String user, String password) throws EventException {
        //make sure the data is there
        if (user==null || password==null) return INVALID_DATA_MISSING;

        //validate the login information
        if (user.equals("santa")) return INVALID_OTHER;
        if (user.equals("klaus")) throw new WhoYouCallingKlausException();
        if (!user.equals("foo")) return INVALID_USER;
        if (!password.equals("bar")) return INVALID_PASSWORD;

        //it's ok!
        return VALID;

    }

    static class WhoYouCallingKlausException extends EventException {
        public WhoYouCallingKlausException() {
            super ("Hey, who are you calling Klaus!!!");
        }
    };

}
