package org.enhydra.barracuda.examples.xmlc;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.data.*;

/**
 * Here's an example of how the component stuff can be implemented
 * in a standalone servlet.
 */
public class CompEx5 extends ComponentGateway {

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //load the localized DOM template
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(CompEx5HTML.class);


        //create a template component and bind it to the views
        BTemplate templateComp = new BTemplate(new SimpleTemplateModel());
        templateComp.setView(new DefaultTemplateView(page.getElementById("SimpleTemplateID")));
        root.addChild(templateComp);


        //create a list component and bind it to the view
        //...list1
        BList blist1 = new BList();
        blist1.setName("BList 1");
        DefaultListModel dlm1 = new DefaultListModel();
        blist1.setModel(dlm1);
        dlm1.add(new BLink("list link ok 1", "http://blah.com/testBLink.html", vc));        
        dlm1.add(new BLink("list link ok 2", "http://blah.com/testBLink.html", vc)); 
        dlm1.add(new BLink("list link ok 3", "http://blah.com/testBLink.html", vc));        
//        blist1.addView(new DefaultView(page.getElementById("SimpleListID1")));
//        blist1.addView(new DefaultView(page.getElementById("SimpleListID2")));
        for (int i=1; i<999; i++) {
            Node n = page.getElementById("SimpleListID"+i);
            if (n==null) break;
            blist1.addView(new DefaultView(n));
        }
        root.addChild(blist1);

        //...list2
        BList blist2 = new BList();
        blist2.setName("BList 2");
        ListModel dlm2 = new DefinitionListModel();
        blist2.setModel(dlm2);
        blist2.addView(new DefaultView(page.getElementById("DefinitionListID1")));
        root.addChild(blist2);


        //create a table component and bind it to the view
        //...table1
        BTable btable1 = new BTable();
        btable1.setName("BTable 1");
        TableModel tm1 = new SimpleTableModel();
        btable1.setModel(tm1);
        btable1.addView(new DefaultTableView(page.getElementById("SimpleTableID1")));
        btable1.addView(new DefaultTableView(page.getElementById("SimpleTableID2")));
        root.addChild(btable1);

        //return the page
        return page;
    }

    /**
     * <p>Get a DOMWriter. By default, we use a DefaultDOMWriter. If
     * you'd like to use something else, override this method.
     *
     * @return a DOMWriter to be used to render the DOM
     */
    public DOMWriter getDOMWriter() {
        //return DefaultDOM writer
        return new DefaultDOMWriter(true);
//        return new DefaultDOMWriter(false);
    }

    //--------------- UserReportModel ------------------------
    /**
     * SimpleTemplateModel
     */
    class SimpleTemplateModel extends AbstractTemplateModel {
        //register the model by name
        public String getName() {
            return "Test1";
        }
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            if (key.startsWith("Link")) {
                BLink blink = new BLink("template link ok "+key.substring(4), "http://blah.com/testBLink1.html", vc);
                return blink;
            } else if (key.equals("TextAndLink")) {
                BLink blink = new BLink("template link ok 5", "http://blah.com/testBLink2.html", vc);
                BInput binput = new BInput(BInput.TEXT, "InputName", "value goes here");
                binput.setDefaultViewContext(vc);   //we must set the default ViewContext since we'll be inlining the component down below
                BToggleButton btoggle = new BToggleButton(BInput.RADIO, "ToggleName", "value goes here", true);
                btoggle.setDefaultViewContext(vc);   //we must set the default ViewContext since we'll be inlining the component down below
                DefaultListModel dlm = new DefaultListModel();
                dlm.add(new DefaultItemMap("Key 1","Value 1"));
                dlm.add(new DefaultItemMap("Key 2","Value 2"));
                dlm.add(new DefaultItemMap("Key 3","Value 3"));
                BSelect bselect = new BSelect(dlm);
                bselect.setDefaultViewContext(vc);   //we must set the default ViewContext since we'll be inlining the component down below
//                return "Inline text, link ("+blink+"), input control "+binput+", toggle button "+btoggle+" and even a select item "+bselect;
                BText btext = new BText("Inline text, link ("+blink+"), input control "+binput+", toggle button "+btoggle+" and even a select item "+bselect);
                btext.setAllowMarkupInText(true);
                return btext;
            } else if (key.equals("BComp1")) {
                //this is not really a link example at all...the purpose is to
                //ensure that we can pass back a simple BComponent (to control
                //visibility/enabled on a block of markup). Note that the component
                //doesn't have to be bound to any node...
                BComponent bcomp = new BComponent();
                bcomp.setVisible(false);
                return bcomp;
            } else return super.getItem(key);
        }
    }


    //--------------- DefinitionList Model -------------------
    class DefinitionListModel extends DefaultListModel {
        public Object getItemAt(int index) {
            ViewContext vc = getViewContext();
            Object[] items = null;
            ElementFactory ef = vc.getElementFactory();
            BLink blDescr = new BLink("definition descr link ok"+(index+1), "http://blah.com/testBLink.html", vc);        
            return new Object[] {"Term "+(index+1), blDescr};
        }

        public int getSize() {return 3;}
    }
    
    
    //--------------- Table Model ----------------------------
    class SimpleTableModel extends AbstractTableModel {
        public Object getItemAt(int row, int col) {
            ViewContext vc = getViewContext();
            if (col==0) {
                return "Row "+(row+1);
            } else if (col==1) {
                return new BLink("table link ok"+(row+1), "http://blah.com/testBLink.html", vc);
            } else {
                return null;
            }    
        }

        public int getRowCount() {return 3;}
        public int getColumnCount() {return 2;}
    }
    
}
