package org.enhydra.barracuda.examples.xmlc;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.plankton.http.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.data.*;

/**
 * Make sure a reusable component can handle nested images within an anchor element
 */
public class CompEx9 extends BTemplateGateway {

    BComponent broot = null;

    Document page = null;
    BTemplate templateComp = null;
    
    //-------------------- BTemplateGateway ----------------------
    /**
     * Return an instance of the template model
     */
    public TemplateModel getTemplateModel() {
        return new LocalTemplateModel();
    }
    
    /**
     * Return an instance of the template class (must implement Document)
     */
    public Class getTemplateClass() {
        return CompEx9HTML.class;
    }
    
    public DOMWriter getDOMWriter() {
        return new DefaultDOMWriter(true);
    }

    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {

        if (templateComp==null) {
            //load the localized DOM template
            page = DefaultDOMLoader.getGlobalInstance().getDOM(getTemplateClass());
        }

            //create a template component and bind it to the views
            templateComp = new BTemplate(getTemplateModel());
            templateComp.setView(new DefaultTemplateView(page.getDocumentElement()));
            root.removeAll();
            root.addChild(templateComp);

        //return the page
        return page;
    }

    protected void handleDefault(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {

        long bmillis = 0;
        long smillis = 0;
        long emillis = 0;
        long elapsed1 = 0;
        long elapsed2 = 0;
        if (logger.isInfoEnabled()) bmillis = System.currentTimeMillis();
        if (logger.isInfoEnabled()) logger.info("Handling incoming HTTP request in "+this);
        
        ViewContext vc = null;
        try {
            //start by figuring out the ViewCapabilities
            if (logger.isDebugEnabled()) logger.debug("Create the ViewContext");
//csc_101701.1            vc = new DefaultViewContext(new ViewCapabilities(req, resp));
            vc = new DefaultViewContext(new ViewCapabilities(req, resp), req, resp);    //csc_101701.1
        
            //create our root component
            if (logger.isDebugEnabled()) logger.debug("Create component root");
            if (broot==null) {
//            if (true) {
                broot = new BComponent();
                broot.setName("Root");
            }
            
            //give the implementation a chance to add any components to the root
            if (logger.isDebugEnabled()) logger.debug("Handling default");
            Document doc = handleDefault(broot, vc, req, resp);
            
            //now init the component
            if (logger.isDebugEnabled()) logger.debug("Invoking initCycle on component hierarchy");
//            broot.initCycle();
broot.validate();
            
            //now render the component
            if (logger.isDebugEnabled()) logger.debug("Rendering component hierarchy");
            if (logger.isDebugEnabled()) smillis = System.currentTimeMillis();
            broot.render(vc);
            if (logger.isDebugEnabled()) elapsed1 = System.currentTimeMillis()-smillis;

            //its possible the implementor may want to recycle children...if so, remove them
            //prior to calling destroy on the root
            if (recycleChildren) {
                if (logger.isDebugEnabled()) logger.debug("Recycling child components");
                List children = broot.getChildren();
                if (children!=null) {
                    for (int i=children.size()-1; i>=0; i--) {
                        broot.removeChild(i);            
                    }
                }
            }

            //now destroy the component
            if (logger.isDebugEnabled()) logger.debug("Invoking destroyCycle on component hierarchy");
//            broot.destroyCycle();

            //csc_102201.1            
            //now adjust the outgoing page (this is critical to make sure we
            //can accurately detect client scripting)
//            ScriptDetector.prepareClientResp(doc, vc);

            //now render the page
            if (logger.isDebugEnabled()) logger.debug("Rendering the DOM");
            if (logger.isInfoEnabled()) smillis = System.currentTimeMillis();
            this.getDOMWriter().write(doc, resp);
            if (logger.isInfoEnabled()) {
                elapsed2 = System.currentTimeMillis()-smillis;
                emillis = System.currentTimeMillis();
            }

            if (logger.isInfoEnabled()) logger.info("Dispatching complete! (rendered in "+(elapsed1)+"/written in "+(elapsed2)+" of "+(emillis-bmillis)+" millis)");

/*
        } catch (ClientSideRedirectException re) {
            //if we get a redirect exception, request the browser to redirect accordingly
//            if (logger.isInfoEnabled()) logger.info("ClientSideRedirectException...redirecting to "+re.getRedirectURL());
//            resp.setStatus(HttpServletResponse.SC_MOVED_TEMPORARILY);
//            resp.setHeader("Location", re.getRedirectURL());

            String url = URLRewriter.encodeRedirectURL(req, resp, re.getRedirectURL());
            if (logger.isInfoEnabled()) logger.info("ClientSideRedirectException...redirecting to "+url);
            resp.sendRedirect(url);    
*/
        } catch (RenderException e) {
            //if we get a RenderException, handle it
            if (logger.isInfoEnabled()) logger.info("Handling RenderException:"+e);
            handleRenderException(e, vc, req, resp);
/*            
        } catch (EventException e) {
            //if we get an EventException, handle it
            if (logger.isInfoEnabled()) logger.info("Handling EventException:"+e);
            handleEventException(e, vc, req, resp);
*/            
        } finally {
            //nop
        }
    }




    //--------------- TemplateModel --------------------------
    /**
     * LocalTemplateModel
     */
    class LocalTemplateModel extends AbstractTemplateModel {
    
        //register the model by name
        public String getName() {return "Ex9";}
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            ViewCapabilities vcap = vc.getViewCapabilities();
            if (key.equals("Link1")) {
                return new BLink("Click me", "CompTest9");
            } else {
                return super.getItem(key);
            }
        }
    }
}
