/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MapStateMap.java,v 1.1 2002/11/02 19:15:53 cryd0221 Exp $
 */
package org.enhydra.barracuda.plankton.data;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * The implementation provides a StateMap bridge to a Map
 * object. By this, we mean that this class allows you to treat an
 * Map as a StateMap. 
 */
public class MapStateMap implements StateMap {

    //private vars
    protected Map map = null;
    
    /**
     * Public constructor.
     *
     * @param map the underlying servlet map structure
     */    
    public MapStateMap(Map imap) {
        map = imap;
    }
    
    /**
     * set a property in this StateMap
     *
     * @param key the key object
     * @param val the value object
     */    
    public void putState(Object key, Object val) {
        map.put(key,val);
    }
    
    /**
     * get a property in this StateMap
     *
     * @param key the key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        return map.get(key);
    }
    
    /**
     * remove a property in this StateMap
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
        return map.remove(key);
    }
    
    /**
     * get a List of the keys for this StateMap (implementation
     * is an ArrayList)
     *
     * @return a List the keys for this StateMap
     */    
    public List getStateKeys() {
        return new ArrayList(map.keySet());
    }
    
    /**
     * get a copy of the underlying Map (in this case, we
     * map all the attributes of the Map structure
     * into a Map and return that)
     *
     * @return a copy of the underlying state Map
     */    
    public Map getStateValues() {
        return map;        
    }
}
