/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: NameSpace.java,v 1.1 2002/11/02 19:15:53 cryd0221 Exp $
 */
package org.enhydra.barracuda.plankton.data;

import java.util.*;

/**
 * This class defines a NameSpace entity (like a key) that can be used to 
 * set/retrieve objects from the ObjectRepository
 */
public class NameSpace {

    protected String name;
    protected static NameSpace globalNameSpace;
    protected static long lastTime = 0;

    /**
     * protected no args constructor (use the factory methods)
     */
    protected NameSpace() {
        this(null);
    }

    protected NameSpace(String iname) {
        if (iname==null) name = "Global";
        else name = iname;
    }

    /**
     * get the name of this NameSpace
     *
     * @return the name of this NameSpace
     */
    public String getName() {
        return name;
    }

    /**
     * return the String represenation of the NameSpace
     *
     * @return the String represenation of the NameSpace
     */
    public String toString() {
        return "NameSpace:"+name;
    }

    /**
     * get the global NameSpace instance
     *
     * @return the global NameSpace instance
     */
    public synchronized static NameSpace globalInstance() {
        if (globalNameSpace==null) globalNameSpace = new NameSpace();
        return globalNameSpace;
    }

    /**
     * get a new NameSpace instance using the default name
     *
     * @return new NameSpace instance using the default name
     */
    public synchronized static NameSpace newInstance() {
        return newInstance("NS");
    }

    /**
     * get a new NameSpace instance using the specified name
     *
     * @param key the namespace key
     * @return the corresponding NameSpace
     */
    public synchronized static NameSpace newInstance(String key) {
        long newTime = 0;
        do {newTime = new java.util.Date().getTime();}
        while (newTime<=lastTime);
        lastTime = newTime;
        NameSpace ns = new NameSpace(key+"_"+newTime);
        return ns;
    }


}