/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PData.java,v 1.1 2002/11/02 19:15:53 cryd0221 Exp $
 */
package org.enhydra.barracuda.plankton.data;

import java.io.*;
import java.util.*;

/**
 * This interface defines the methods required to establish 
 * a heirarchical relationship between objects.
 */
public interface PData extends StateMap, Cloneable, Serializable {

    /**
     * set the object's parent
     *
     * @param   p  
     */
    public void setParent(PData p);

    /**
     * get the objects parent (null if root)
     *
     * @return the PData object
     */
    public PData getParent();

    /**
     * get the root parent by chaining back up the heirarchy until we find 
     * the highest PData object in the heirarchy.
     *
     * @return the root PData object
     */
    public PData getRootParent();
    
    /**
     * Do we want to inherit parents. Defaults to true. This indicates that 
     * when an object is added to this data collection, if it implements
     * PData, it should automatically inherit the current object as its 
     * parent. You would typically set this to false if you were going
     * to be storing PData "mini-hierarchies" within a larger PData 
     * hierarchy
     * 
     * @param val true if we should inherit parents
     */
    public void setInheritParents(boolean val);
    
    /**
     * Return true if we are inheriting parents.
     * 
     * @return true if we are inheriting parents
     */
    public boolean isInheritParents();
}
