/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: NestableException.java,v 1.1 2002/11/02 19:15:53 cryd0221 Exp $
 */
package org.enhydra.barracuda.plankton.exceptions;

/**
 * This class simply defines a NestableException.
 * A NestableException can contain other exceptions
 * (which may have triggered this exception). The
 * getRootException method provides a means to find the
 * root exception in a NestableException chain.
 */
public class NestableException extends Exception {

    private Exception baseException = null;

    /**
     * The noargs public contructor for NestableException
     */
    public NestableException () {
        this("Unidentified Exception");
    }

    /**
     * The public contructor for NestableException
     *
     * @param s a String describing the exception
     */
    public NestableException (String s) {
        this(s, null);
    }

    /**
     * The public contructor for NestableException
     *
     * @param s a String describing the exception
     * @param iError the Error code associated with this exception
     */
    public NestableException (String s, Exception ibaseException) {
        super(s);
        baseException = ibaseException;
    }

    /**
     * get the BaseException behind this exception
     *
     * @return Exception baseException
     */
    public Exception getBaseException () {
        return baseException;
    }

    /**
     * get the RootException behind this exception. Will look for
     * a baseException, and if it happens to be an instance of a
     * NestableException, will recursively work deeper until it finds
     * the exception which caused it all
     *
     * @param ne a NestableException for which we wish to find
     *      the root exception
     * @return Exception the exception which caused it all
     */
    public synchronized static Exception getRootException (NestableException ne) {
        Exception e = ne.getBaseException();
        if (e!=null && e instanceof NestableException) return getRootException ((NestableException) e);
        return ne;
    }
}