/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LocaleTestSuiteSetup.java,v 1.3 2003/01/21 05:43:03 jacobk Exp $
 */
package org.enhydra.barracuda.testbed;

import junit.framework.Test;
import junit.extensions.TestSetup;

import java.util.*;

//
// a test setup that will set the default locale in setUp()
// and reset it in tearDown(). Used for running test cases in different locales
// (see TestDateValidator.java and TestDateRangeValidator.java)
//
public class LocaleTestSuiteSetup extends TestSetup {

    //===========================================================================
    // CONSTRUCTORS
    //===========================================================================

    /**
     * Default constructor
     *
     * @param theTest a Test object
     * @param theNewLocale the preferred local with which to test
     */
    public LocaleTestSuiteSetup(Test theTest, Locale theNewLocale) {
        super(theTest);
        this.myNewLocale = theNewLocale;
    }

    //===========================================================================
    // QUERY METHODS
    //===========================================================================

    //===========================================================================
    // MUTATOR METHODS
    //===========================================================================

    public void setUp() {
        this.myOriginalLocale = Locale.getDefault();
        Locale.setDefault(this.myNewLocale);
    }

    public void tearDown() {
        Locale.setDefault(this.myOriginalLocale);
    }

    //===========================================================================
    // HELPER METHODS
    //===========================================================================

    //===========================================================================
    // DATA MEMBERS
    //===========================================================================

    private Locale myOriginalLocale = null;
    private Locale myNewLocale = null;

    //===========================================================================
    // STATIC DATA MEMBERS
    //===========================================================================

}
