/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestBasicComponentsRender.java,v 1.9 2002/11/22 23:10:23 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import junit.framework.*;
import org.apache.log4j.*;
import org.apache.log4j.xml.*;

//import org.enhydra.barracuda.plankton.*;
//import org.enhydra.barracuda.plankton.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.*;
import org.enhydra.barracuda.testbed.*;
import org.enhydra.barracuda.testbed.servlet.*;


/**
 * This test verifies the basic component renderings
 */
public class TestBasicComponentsRender extends DefaultTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestBasicComponentsRender.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestBasicComponentsRender(String name) {
        super(name);
    }

    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Simple test to verify that the component renders as expected
     */
    public void testBasicRender() throws IOException, RenderException {
        //render the doc
        if (logger.isInfoEnabled()) logger.info("testing basic render");
        CompEx1 cg = new CompEx1();
        BComponent root = new BComponent();
        root.initCycle();

        //build a sample view capabilities object
        MockHttpServletRequest req = new MockHttpServletRequest();
        req.setHeader("User-Agent","Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0)");
        ViewCapabilities vcap = new ViewCapabilities(req, null);
//System.out.println(vcap.toString());        

        //create the component amd render
        ViewContext vc = new DefaultViewContext(vcap, req, null);
        Document doc = cg.handleDefault(root, vc, req, null);
        root.render(vc);
        root.destroyCycle();
        DefaultDOMWriter ddw = new DefaultDOMWriter(false);
        StringWriter sw = new StringWriter(2000);
        ddw.write(doc, sw);
        String html = sw.toString();
//        ddw.write(doc, System.out);

        //assert that there are no 'link failed' messages in the doc
        int fpos = html.indexOf("render failed");
        assertTrue("BLink render err: "+(fpos>20 ? html.substring(fpos-20,fpos+20) : "??"), fpos<0);

    }
}
