/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestDefaultEventBroker.java,v 1.1 2003/01/21 14:17:58 jacobk Exp $
 */
package org.enhydra.barracuda.core.event;

import java.util.*;

import junit.framework.*;
import org.apache.log4j.*;

import org.enhydra.barracuda.plankton.data.*;
import org.enhydra.barracuda.testbed.*;

//csc_121302.1 - created
/**
 * Unit tests for DefaultEventBroker (make sure we can register and look up ids/events
 * correctly
 */
public class TestDefaultEventBroker extends DefaultTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestDefaultEventBroker.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestDefaultEventBroker(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Simple test to verify that the component renders as expected
     */
    public void testListeners() throws InvalidClassException {
        if (logger.isInfoEnabled()) logger.info("testing event broker listeners");

        //create the event broker
        DefaultEventBroker eb = new DefaultEventBroker(null, ".event");
        List lAliases = null;
        String id = null;
        String result = null;
        
        //create some aliases 
        String classes[] = new String[] {
            "foo.blah.event.Test",
            "foo.blah.event.Blarney",
            "foo.blah.event2.Test"
        };
        for (int i=0; i<classes.length; i++) {
            eb.addAliases(classes[i], eb.getAliases(classes[i]), eb.eventXref);
        }       

        //now see what we have
/*        
        System.out.println ("");
        System.out.println ("Check @ point 1:");
        CollectionsUtil.printStackTrace(eb.eventXref, System.out);
*/        

        //now see if we get matches
        id = "Blarney";
        result = eb.matchEventClass(id);
        assertTrue("failed to find id:"+id+", got:"+result, classes[1].equals(result));
        id = "event2.Test";
        result = eb.matchEventClass(id);
        assertTrue("failed to find id:"+id+", got:"+result, classes[2].equals(result));
        id = "Test";
        try {
            result = eb.matchEventClass(id);
            fail("failed to throw exception on id:"+id+", got:"+result);
        } catch (InvalidClassException e) {
            //noop - this is what we expected
        }
        id = "BLARNEY";  //test upper case
        result = eb.matchEventClass(id);
        assertTrue("failed to find id:"+id+", got:"+result, classes[1].equals(result));
        id = "blarney";  //test lower case
        result = eb.matchEventClass(id);
        assertTrue("failed to find id:"+id+", got:"+result, classes[1].equals(result));
        id = "fOo.BlAh.EvEnT.bLaRnEy";  //test mixed case
        result = eb.matchEventClass(id);
        assertTrue("failed to find id:"+id+", got:"+result, classes[1].equals(result));
    }
}
