/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestEqualsValidator.java,v 1.9 2002/11/22 23:10:23 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.plankton.data.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.testbed.*;


/**
 * This test verifies that the EqualsValidator works correctly.
 */
public class TestEqualsValidator extends ValidatorTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestEqualsValidator.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestEqualsValidator(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Test String types
     */
    public void testString() {
        //create a form validator and excercise it
        String s1 = "foo";
        EqualsValidator v1 = new EqualsValidator(s1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.STRING);

        // ilc_031902.1_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();
        sm.putState("valid equals1", null);
        sm.putState("valid equals2", "foo");
        assertAllValid(v1, sm, FormType.STRING);

        sm = new DefaultStateMap();
        sm.putState("invalid equals1", "foo2");
        assertAllInvalid(v1, sm, FormType.STRING);

        sm = new DefaultStateMap();
        sm.putState("valid equal null", null);
        assertAllValid(v2, sm, FormType.STRING);

        sm = new DefaultStateMap();
        sm.putState("invalid equal null", "foo");
        assertAllInvalid(v2, sm, FormType.STRING);
        // ilc_0311902.1_end


        // ilc_031902.2_start
        // use a statemap above
        /*
        //v1
        assertValid("Error validating foo==foo", v1, el, s1);
        assertValid("Error validating foo.equals(foo)", v1, el, "foo");
        assertInvalid("Error invalidating foo.equals(foo2)", v1, el, "foo2");
        assertInvalid("Error invalidating foo.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null==foo2", v2, el, "foo2");
        */
        // ilc_031902.2_end

    }
    
    /**
     * Test Boolean types
     */
    public void testBoolean() {
        //create a form validator and excercise it
        Boolean bT = new Boolean(true);
        Boolean bF = new Boolean(false);
        EqualsValidator v1 = new EqualsValidator(bT);
        EqualsValidator v2 = new EqualsValidator(bF);
        EqualsValidator v3 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.BOOLEAN);

        // ilc_031902.3_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();
        sm.putState("valid boolean equals true 1", null);
        sm.putState("valid boolean equals true 2", bT);
        sm.putState("valid boolean equals true 3", "true");
        sm.putState("valid boolean equals true 4", "yes");
        assertAllValid(v1, sm, FormType.BOOLEAN);

        sm = new DefaultStateMap();
        sm.putState("invalid boolean equals true 1", bF);
        sm.putState("invalid boolean equals true 2", "false");
        sm.putState("invalid boolean equals true 3", "no");
        assertAllInvalid(v1, sm, FormType.BOOLEAN);

        sm = new DefaultStateMap();
        sm.putState("valid boolean equals false 1", null);
        sm.putState("valid boolean equals false 2", bF);
        sm.putState("valid boolean equals false 3", "false");
        sm.putState("valid boolean equals false 4", "no");
        assertAllValid(v2, sm, FormType.BOOLEAN);

        sm = new DefaultStateMap();
        sm.putState("invalid boolean equals false 1", bT);
        sm.putState("invalid boolean equals false 2", "true");
        sm.putState("invalid boolean equals false 2", "yes");
        assertAllInvalid(v2, sm, FormType.BOOLEAN);

        sm = new DefaultStateMap();
        sm.putState("valid boolean equals null 1", null);
        assertAllValid(v3, sm, FormType.BOOLEAN);

        sm = new DefaultStateMap();
        sm.putState("invalid boolean equals null 1", bT);
        sm.putState("invalid boolean equals null 2", bF);
        sm.putState("invalid boolean equals null 3", "true");
        sm.putState("invalid boolean equals null 4", "false");
        sm.putState("invalid boolean equals null 5", "no");
        sm.putState("invalid boolean equals null 6", "yes");
        assertAllInvalid(v3, sm, FormType.BOOLEAN);

        // ilc_031902.3 end


        // ilc_031902.4_start
        // use a statemap above
        /*
        //v1
        assertValid("Error validating true==true", v1, el, bT);                        
        assertValid("Error validating true.equals(true)", v1, el, new Boolean(true));
        assertInvalid("Error invalidating true.equals(false)", v1, el, bF);            
        assertInvalid("Error invalidating true.equals(null)", v1, el, null);            

        //v2
        assertValid("Error validating false==false", v2, el, bF);                    
        assertValid("Error validating false.equals(false)", v2, el, new Boolean(false));
        assertInvalid("Error invalidating false.equals(true)", v2, el, bT);    
        assertInvalid("Error invalidating false.equals(null)", v2, el, null);    

        //v3
        assertValid("Error validating null==null", v3, el, null);            
        assertInvalid("Error invalidating null==true", v3, el, bT);            
        assertInvalid("Error invalidating null==false", v3, el, bF);            
        */
        // ilc_031902.4_end
    }
    
    /**
     * Test Integer types
     */
    public void testInteger() {
        //create a form validator and excercise it
        Integer i1 = new Integer(1000);
        EqualsValidator v1 = new EqualsValidator(i1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.INTEGER);

        // ilc_031902.5_start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();
        sm.putState("valid integer equals 1000 1", null);
        sm.putState("valid integer equals 1000 2", new Integer(1000));
        assertAllValid(v1, sm, FormType.INTEGER);

        sm = new DefaultStateMap();
        sm.putState("invalid integer equals 1000", new Integer(2000));
        assertAllInvalid(v1, sm, FormType.INTEGER);

        sm = new DefaultStateMap();
        sm.putState("valid integer equals null", null);
        assertAllValid(v2, sm, FormType.INTEGER);

        sm = new DefaultStateMap();
        sm.putState("invalid integer equals null", i1);
        assertAllInvalid(v2, sm, FormType.INTEGER);

        // ilc_031902.5 end

        // ilc_031902.6 start
        // use statemap above
        /*
        //v1
        assertValid("Error validating i1==i1", v1, el, i1);
        assertValid("Error validating i1.equals(1000)", v1, el, new Integer(1000));
        assertInvalid("Error invalidating i1.equals(2000)", v1, el, new Integer(2000));
        assertInvalid("Error invalidating i1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(2000)", v2, el, new Integer(2000));
      */
      // ilc_031902.6 end
    }

    /**
     * Test Date types
     */
    public void testDate() {
        //create a form validator and excercise it
        Date d1 = new Date();
        Date d2 = new Date(d1.getTime());
        Date d3 = new Date(65000);
        EqualsValidator v1 = new EqualsValidator(d1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.DATE);

        // ilc_031902.7 start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();
        sm.putState("1 valid date equals " + d1, null);
        sm.putState("2 valid date equals " + d1, d1);
        sm.putState("2 valid date equals " + d1, d2);
        assertAllValid(v1, sm, FormType.DATE);

        sm = new DefaultStateMap();
        sm.putState("1 invalid date equals " + d1, d3);
        assertAllInvalid(v1, sm, FormType.DATE);

        sm = new DefaultStateMap();
        sm.putState("1 valid date equals null", null);
        assertAllValid(v2, sm, FormType.DATE);

        sm = new DefaultStateMap();
        sm.putState("1 invalid date equals null", d3);
        assertAllInvalid(v2, sm, FormType.DATE);

        // ilc_031902.7 end

        // ilc_031902.8 start
        // use statemap above
        /*
        //v1
        assertValid("Error validating d1==d1", v1, el, d1);
        assertValid("Error validating d1.equals(d2)", v1, el, d2);
        assertInvalid("Error invalidating d1.equals(d3)", v1, el, d3);
        assertInvalid("Error invalidating d1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(d3)", v2, el, d3);
        */
        // ilc_031902.8 end
    }

    /**
     * Test Long types
     */
    public void testLong() {
        //create a form validator and excercise it
        Long l1 = new Long(1000);
        EqualsValidator v1 = new EqualsValidator(l1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.LONG);

        // ilc_031902.9 start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();
        sm.putState("1 valid long equals " + l1, null);
        sm.putState("2 valid long equals " + l1, new Long(1000));
        sm.putState("3 valid long equals " + l1, "1000");
        assertAllValid(v1, sm, FormType.LONG);

        sm = new DefaultStateMap();
        sm.putState("1 invalid long equals " + l1, new Long(2000));
        sm.putState("1 invalid long equals " + l1, "2000");
        assertAllInvalid(v1, sm, FormType.LONG);

        sm = new DefaultStateMap();
        sm.putState("1 valid long equals null", null);
        sm.putState("1 valid long equals null", "");
        sm.putState("1 valid long equals null", "  ");
        assertAllValid(v2, sm, FormType.LONG);

        sm = new DefaultStateMap();
        sm.putState("1 invalid long equals null", l1);
        assertAllInvalid(v2, sm, FormType.LONG);


        // ilc_031902.9 end

        // ilc_031902.10 start
        // use statemap above
        /*
        //v1
        assertValid("Error validating l1==l1", v1, el, l1);
        assertValid("Error validating l1.equals(1000)", v1, el, new Long(1000));
        assertInvalid("Error invalidating l1.equals(2000)", v1, el, new Long(2000));
        assertInvalid("Error invalidating l1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(2000)", v2, el, new Long(2000));
        */
        // ilc_031902.10 end
    }

    /**
     * Test Short types
     */
    public void testShort() {
        //create a form validator and excercise it
        Short s1 = new Short((short) 1000);
        EqualsValidator v1 = new EqualsValidator(s1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.SHORT);

        // ilc_031902.11 start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();
        sm.putState("1 valid short equals " + s1, null);
        sm.putState("2 valid short equals " + s1, new Short((short)1000));
        sm.putState("3 valid short equals " + s1, "1000");
        assertAllValid(v1, sm, FormType.SHORT);

        sm = new DefaultStateMap();
        sm.putState("1 invalid short equals " + s1, new Short((short)2000));
        sm.putState("2 invalid short equals " + s1, "2000");
        assertAllInvalid(v1, sm, FormType.SHORT);

        sm = new DefaultStateMap();
        sm.putState("1 valid short equals null", null);
        sm.putState("2 valid short equals null", "");
        sm.putState("3 valid short equals null", "  ");
        assertAllValid(v2, sm, FormType.SHORT);

        sm = new DefaultStateMap();
        sm.putState("1 invalid short equals null", s1);
        assertAllInvalid(v2, sm, FormType.SHORT);
        // ilc_031902.11 end

        // ilc_031902.12 start
        // use statemap above
        /*
        //v1
        assertValid("Error validating s1==s1", v1, el, s1);
        assertValid("Error validating s1.equals(1000)", v1, el, new Short((short) 1000));
        assertInvalid("Error invalidating s1.equals(2000)", v1, el, new Short((short) 2000));
        assertInvalid("Error invalidating s1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(2000)", v2, el, new Short((short) 2000));
        */
        // ilc_031902.12 end
    }

    /**
     * Test Double types
     */
    public void testDouble() {
        //create a form validator and excercise it
        Double d1 = new Double(1000);
        EqualsValidator v1 = new EqualsValidator(d1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.DOUBLE);

        // ilc_031902.13 start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();
        sm.putState("1 valid double equals " + d1, null);
        sm.putState("2 valid double equals " + d1, new Double(1000));
        sm.putState("3 valid double equals " + d1, "1000");
        assertAllValid(v1, sm, FormType.DOUBLE);

        sm = new DefaultStateMap();
        sm.putState("1 invalid double equals " + d1, new Double(2000));
        sm.putState("2 invalid double equals " + d1, "2000");
        assertAllInvalid(v1, sm, FormType.DOUBLE);

        sm = new DefaultStateMap();
        sm.putState("1 valid double equals null", null);
        sm.putState("2 valid double equals null", "");
        sm.putState("3 valid double equals null", "  ");
        assertAllValid(v2, sm, FormType.DOUBLE);

        sm = new DefaultStateMap();
        sm.putState("1 invalid double equals null", d1);
        assertAllInvalid(v2, sm, FormType.DOUBLE);
        // ilc_031902.13 end

        // ilc_031902.14 start
        // use statemap above
        /*
        //v1
        assertValid("Error validating d1==d1", v1, el, d1);
        assertValid("Error validating d1.equals(1000)", v1, el, new Double(1000));
        assertInvalid("Error invalidating d1.equals(2000)", v1, el, new Double(2000));
        assertInvalid("Error invalidating d1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(2000)", v2, el, new Double(2000));
        */
        // ilc_031902.14 end
    }

    /**
     * Test Float types
     */
    public void testFloat() {
        //create a form validator and excercise it
        Float f1 = new Float(1000);
        EqualsValidator v1 = new EqualsValidator(f1);
        EqualsValidator v2 = new EqualsValidator(null);
        DefaultFormElement el = new DefaultFormElement("key", FormType.FLOAT);

        // ilc_031902.15 start
        // use a StateMap for testing to make sure elements go through mapping
        DefaultStateMap sm = new DefaultStateMap();
        sm.putState("1 valid float equals " + f1, null);
        sm.putState("2 valid float equals " + f1, new Float(1000));
        sm.putState("3 valid float equals " + f1, "1000");
        assertAllValid(v1, sm, FormType.FLOAT);

        sm = new DefaultStateMap();
        sm.putState("1 invalid float equals " + f1, new Float(2000));
        sm.putState("2 invalid float equals " + f1, "2000");
        assertAllInvalid(v1, sm, FormType.FLOAT);

        sm = new DefaultStateMap();
        sm.putState("1 valid float equals null", null);
        sm.putState("2 valid float equals null", "");
        sm.putState("3 valid float equals null", "  ");
        assertAllValid(v2, sm, FormType.FLOAT);

        sm = new DefaultStateMap();
        sm.putState("1 invalid float equals null", f1);
        assertAllInvalid(v2, sm, FormType.FLOAT);
        // ilc_031902.15 end

        // ilc_031902.16 start
        // use statemap above
        /*
        //v1
        assertValid("Error validating f1==f1", v1, el, f1);
        assertValid("Error validating f1.equals(1000)", v1, el, new Float(1000));
        assertInvalid("Error invalidating f1.equals(2000)", v1, el, new Float(2000));
        assertInvalid("Error invalidating f1.equals(null)", v1, el, null);

        //v2
        assertValid("Error validating null==null", v2, el, null);
        assertInvalid("Error invalidating null.equals(2000)", v2, el, new Float(2000));
        */
        // ilc_031902.16 end
    }
    
}
