/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestPHashMap.java,v 1.2 2002/11/22 23:10:23 cryd0221 Exp $
 */
package org.enhydra.barracuda.plankton.data;

import java.util.*;

import junit.framework.*;

//import org.enhydra.barracuda.plankton.data.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.testbed.*;


/**
 * Test case for PHashMap
 */
public class TestPHashMap extends PMapTestCases {
    //common vars (customize for every test class)
    static {
        testClass = TestPHashMap.class.getName();
        logger = Logger.getLogger("test."+testClass);
    }
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestPHashMap(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    //NOTE: most of the individual tests are actually inherited 
    //from the parent class

    /**
     * Verify the clone (we have to test this here instead
     * the clone method is not part of the Map interface...we need
     * a concrete implementation in order to be able to reference it)
     */
    public void testClone() {
        if (logger.isInfoEnabled()) logger.info("testing clone()");
        PHashMap pmap1 = null;
        PHashMap pmap2 = null;

        //test an empty list
        pmap1 = (PHashMap) this.getPMapInstance();
        pmap2 = (PHashMap) pmap1.clone();
        assertTrue("Cloned obj==source obj at 1", pmap1!=pmap2);
        assertEquals("Clone check 1a failed", pmap1, pmap2);
        assertEquals("Clone check 1b failed", pmap2, pmap1);

        //test a list with some items
        pmap1 = (PHashMap) this.getPMapInstance();
        pmap1.put("key1", "foo1");
        pmap1.put("key2", "foo2");
        pmap1.put("key3", "foo3");
        pmap1.put("key4", new Integer(99));
        pmap1.put("key5", null);
        pmap2 = (PHashMap) pmap1.clone();
        assertTrue("Cloned obj==source obj at 2", pmap1!=pmap2);
        assertEquals("Clone check 2a failed", pmap1, pmap2);
        assertEquals("Clone check 2b failed", pmap2, pmap1);
        
        //test a list with some PData items
        pmap1 = (PHashMap) this.getPMapInstance();
        pmap1.put("key1", "foo1");
        PMap pmTmp = this.getPMapInstance();
        pmTmp.put("key2a", "blah 1");
        pmTmp.put("key2b", "blah 2");
        pmap1.put("key2", pmTmp);
        pmap2 = (PHashMap) pmap1.clone();
        assertTrue("Cloned obj==source obj at 3", pmap1!=pmap2);
        assertEquals("Clone check 3a failed", pmap1, pmap2);
        assertEquals("Clone check 3b failed", pmap2, pmap1);
    }


    //-------------------- Abstract methods ----------------------
    public StateMap getStateMap() {
        return getPMapInstance();
    }

    public PData getPDataInstance() {
        return getPMapInstance();
    }

    public PList getPListInstance() {
        return new PArrayList();
    }

    public PMap getPMapInstance() {
        return new PHashMap();
    }
}
